import { Metadata, Http } from '@zeedhi/core';
import { Modal, ModalService, Grid, Form, CodeEditor, LoadingService, AlertService, DialogService } from '@zeedhi/common';
import { TekGrid } from "@zeedhi/teknisa-components-common";
import { CustomAlertService } from '@/Services/CustomAlertService';
import { Highlight } from '@zeedhi/zd-highlight-common';
import Prism from 'prismjs';
import 'prismjs/components/prism-markup-templating.min';
import 'prismjs/components/prism-php.min';

export class ManagementController {
    public loadLanguage() {
        const { code, language } = Metadata.getInstance<Highlight>('query');
        return Prism.highlight(code, Prism.languages[language], language);
    }
    //  Metodos de dialog
    public showRegenDialog(type: string) {
        const dialogEvents: any = {
            click: ({ component }: any) => {
                DialogService.hide();
                if (component.name === "sim") {
                    this.regenGrid(type);
                }
            }
        };
        DialogService.show({
          name: 'delete-dialog',
          type: 'warning',
          title: ' ',
          text: 'Esse procedimento vai gerar todo o relatório que pode demorar alguns minutos! Deseja continuar?',
          buttons: [
            { name: 'não', component: 'ZdButton', label: 'NÃO', events: dialogEvents, outline: true },
            { name: 'sim', component: 'ZdButton', label: 'SIM', events: dialogEvents },
          ],
        });
    }
    //  Metodos de Modal
    public modalOptions: any = {
        "agendadas": "management/AgendasModal/AgendasModal",
        "importacao": 'management/ImportacaoModal/ImportacaoModal',
        "notification": 'management/NotificationModal/NotificationModal'
    };
    public gridRelations: any = {
        "gridImportacao": {
            "modal": "importacao",
            "regen": "importLogImportation"
        },
        "gridColetaAgendada": {
            "modal": "agendadas",
            "regen": "notificationImportationFail"
        }
    };
    public currentModal!: Modal;
    public async createModal(type: any) {
        const path = this.modalOptions[type];
        this.currentModal = await ModalService.createFromJson(path, true);
    }
    public destroyModal() {
        this.currentModal?.destroy();
    }
    public closeModal() {
        this.currentModal.hide();
    }
    //  Metodos do Formulario
    protected modalData: any;
    public get currentRowId() {
        return this.modalData._id;
    }
    public formOnMounted({ component }: any) {
        const form = component;
        const formValues = form.value;
        const data = this.modalData;
        const specialKeys = ["logs"];
        Object.keys(formValues).filter((key: string) => !specialKeys.includes(key)).forEach((key: string) => { formValues[key] = data[key]; });
        if (data.logs?.length > 0) {
            formValues.logs = data.logs[0].logs.map(({type, message}: any) => "Type: " + type + "\nMessage: " + message).join("\n\n");
        }
        const code = this.modalData.query || this.modalData.QUERY || "";
        form.getChildInstance("query").code = code;
        form.getChildInstance("queryCode").staticCode = code;
    }
    public async openDetailModal({ component, row }: any) {
        this.destroyModal();
        await this.createModal(this.gridRelations[component.name].modal);
        this.modalData = row === undefined ? {} : row;
        this.currentModal.show();
    }
    public copyQueryOnModal() {
        const form = this.currentModal.getChildInstance<Form>("detailForm");
        form.getChildInstance<CodeEditor>("queryCode").copyToClipboard();
        CustomAlertService.successAlert("Query copiada!");
    }
    public regenImport() {
        this.showRegenDialog("gridImportacao");
    }
    public regenAgenda() {
        this.showRegenDialog("gridColetaAgendada");
    }
    public async regenGrid(type: string) {
        const url = this.gridRelations[type].regen;
        LoadingService.show("Aguarde...");
        try {
            const res = await Http.post(url, {
                "filter": [],
                "requestType": "FilterData"
            });
            if (res.data?.messages?.length > 0 && res.data.messages[0].message) {
                CustomAlertService.warningAlert(res.data.messages[0].message);
            }
        } catch (error) {
            console.error(error);
            CustomAlertService.errorAlert("Ocorreu um erro.");
        }
        LoadingService.hide();
        return this.reloadGrid(type);
    }
    //  Notification Modal
    public notificationFormOnMounted({ component }: any) {
        const form = component;
        const formValues = form.value;
        const data = this.modalData;
        Object.keys(formValues).forEach((key: string) => { formValues[key] = data[key].join(","); });
    }
    public async openNotificationModal() {
        this.destroyModal();
        await this.createModal("notification");
        try {
            const res = await Http.get("configImportationFail");
            this.modalData = res.data.data[0];
            this.currentModal.show();
        } catch (error) {
            console.error(error);
            CustomAlertService.errorAlert("Ocorreu um erro.");
        }
    }
    public async saveNotificationModal() {
        const form = this.currentModal.getChildInstance<Form>("notificationForm");
        const values = form.value;
        const content = {
            "EMAIL_SAAS": values.EMAIL_SAAS.split(','),
            "EMAIL_HOST": values.EMAIL_HOST.split(','),
            "EMAIL_LOCAL": values.EMAIL_LOCAL.split(','),
            "changed": true,
            "_id": this.modalData._id
        };
        try {
            const res = await Http.post("configImportationFail/save", {
                "requestType": "DataSet",
                "disableLoader": false,
                "dataset": [content]
            });
            CustomAlertService.saveSuccessAlert();
        } catch (error) {
            console.error(error);
            CustomAlertService.errorAlert("Ocorreu um erro.");
        }
        this.closeModal();
    }
    //  Filter
    public async handleBeforeFilter({ component }: any) {
        const datasource = component.datasource;
        const filter = datasource.dynamicFilter;
        Object.entries(filter).forEach(([k, filters]: any) => {
            filter[k] = filters.filter(({value}: any) => !(value instanceof Array && value.length === 0));
            if (filter[k].length === 0) {
                delete filter[k];
            }
        });
    }
    public reloadGrid(type: string) {
        const grid = Metadata.getInstance<TekGrid>(type);
        return grid.reload();
    }
}
